@@ -1,11 +1,14 @@
 """Routines related to PyPI, indexes"""
 
+# The following comment should be removed at some point in the future.
+# mypy: strict-optional=False
+
 import enum
 import functools
 import itertools
 import logging
 import re
-from typing import TYPE_CHECKING, FrozenSet, Iterable, List, Optional, Set, Tuple, Union
+from typing import FrozenSet, Iterable, List, Optional, Set, Tuple, Union
 
 from pip._vendor.packaging import specifiers
 from pip._vendor.packaging.tags import Tag
@@ -36,9 +39,6 @@ from pip._internal.utils.misc import build_netloc
 from pip._internal.utils.packaging import check_requires_python
 from pip._internal.utils.unpacking import SUPPORTED_EXTENSIONS
 
-if TYPE_CHECKING:
-    from pip._vendor.typing_extensions import TypeGuard
-
 __all__ = ["FormatControl", "BestCandidateResult", "PackageFinder"]
 
 
@@ -251,7 +251,7 @@ class LinkEvaluator:
 
 def filter_unallowed_hashes(
     candidates: List[InstallationCandidate],
-    hashes: Optional[Hashes],
+    hashes: Hashes,
     project_name: str,
 ) -> List[InstallationCandidate]:
     """
@@ -540,7 +540,6 @@ class CandidateEvaluator:
                 binary_preference = 1
             if wheel.build_tag is not None:
                 match = re.match(r"^(\d+)(.*)$", wheel.build_tag)
-                assert match is not None, "guaranteed by filename validation"
                 build_tag_groups = match.groups()
                 build_tag = (int(build_tag_groups[0]), build_tag_groups[1])
         else:  # sdist
@@ -943,46 +942,43 @@ class PackageFinder:
                 "No matching distribution found for {}".format(req)
             )
 
-        def _should_install_candidate(
-            candidate: Optional[InstallationCandidate],
-        ) -> "TypeGuard[InstallationCandidate]":
-            if installed_version is None:
-                return True
-            if best_candidate is None:
-                return False
-            return best_candidate.version > installed_version
+        best_installed = False
+        if installed_version and (
+            best_candidate is None or best_candidate.version <= installed_version
+        ):
+            best_installed = True
 
         if not upgrade and installed_version is not None:
-            if _should_install_candidate(best_candidate):
+            if best_installed:
+                logger.debug(
+                    "Existing installed version (%s) is most up-to-date and "
+                    "satisfies requirement",
+                    installed_version,
+                )
+            else:
                 logger.debug(
                     "Existing installed version (%s) satisfies requirement "
                     "(most up-to-date version is %s)",
                     installed_version,
                     best_candidate.version,
                 )
-            else:
-                logger.debug(
-                    "Existing installed version (%s) is most up-to-date and "
-                    "satisfies requirement",
-                    installed_version,
-                )
             return None
 
-        if _should_install_candidate(best_candidate):
+        if best_installed:
+            # We have an existing version, and its the best version
             logger.debug(
-                "Using version %s (newest of versions: %s)",
-                best_candidate.version,
+                "Installed version (%s) is most up-to-date (past versions: %s)",
+                installed_version,
                 _format_versions(best_candidate_result.iter_applicable()),
             )
-            return best_candidate
+            raise BestVersionAlreadyInstalled
 
-        # We have an existing version, and its the best version
         logger.debug(
-            "Installed version (%s) is most up-to-date (past versions: %s)",
-            installed_version,
+            "Using version %s (newest of versions: %s)",
+            best_candidate.version,
             _format_versions(best_candidate_result.iter_applicable()),
         )
-        raise BestVersionAlreadyInstalled
+        return best_candidate
 
 
 def _find_name_version_sep(fragment: str, canonical_name: str) -> int:
